#include "CSingleLinkedList.h"


//КОНСТРУКТОРИ/ДЕСТРУКТОР
CSingleLinkedList::CSingleLinkedList::CSingleLinkedList(const CSingleLinkedList& other) : head(nullptr), Size(other.Size) {
    if (other.head) {
        head = std::make_unique<Node>(Node{ other.head->data, nullptr });
        Node* curr = head.get();
        Node* otherCurr = other.head->next.get();
        while (otherCurr) {
            curr->next = std::make_unique<Node>(Node{ otherCurr->data, nullptr });
            curr = curr->next.get();
            otherCurr = otherCurr->next.get();
        }
    }
}

CSingleLinkedList::CSingleLinkedList(CSingleLinkedList&& other) noexcept : head(nullptr), Size(0) {//переміщення
    std::swap(head, other.head);
    std::swap(Size, other.Size);
}



// ОПЕРАТОРИ ПРИСВОЄННЯ/ПЕРЕМІЩЕНН
CSingleLinkedList& CSingleLinkedList::operator=(const CSingleLinkedList& other) {
    if (this != &other) {
        head.reset(); // очищаємо поточний список
        Node* prev = nullptr;
        Node* otherCurrent = other.head.get();
        while (otherCurrent) {
            std::unique_ptr<Node> newNode = std::make_unique<Node>(Node{ otherCurrent->data, nullptr });
            if (!head) {
                head = std::move(newNode);
            }
            else {
                prev->next = std::move(newNode);
            }
            prev = newNode.get();
            otherCurrent = otherCurrent->next.get();
        }
        Size = other.Size;
    }
    return *this;
}

CSingleLinkedList& CSingleLinkedList::operator=(CSingleLinkedList&& other) noexcept {
    if (this != &other) {
        head.reset(); // Очищаємо поточний список
        Size = 0;
        // Обмінюємо вміст з іншим списком за допомогою std::swap
        std::swap(head, other.head);
        std::swap(Size, other.Size);
    }
    return *this;
}




//МЕТОДИ
CSingleLinkedList CSingleLinkedList::addToBegin(double value) {
    std::unique_ptr<Node> newNode = std::make_unique<Node>(Node{ value, nullptr });
    newNode->next = std::move(head);
    head = std::move(newNode);
    Size++;
    return *this;
}


CSingleLinkedList CSingleLinkedList::removeFirst() {
    if (head) {
        head = std::move(head->next);
        Size--;
    }
    return *this;
}

CSingleLinkedList CSingleLinkedList::addAfter(double value, int AfterElem) {
    if (AfterElem < 0 || AfterElem >= Size) {
        return *this; // Якщо позиція після якої потрібно вставити недійсна, повертаємо поточний об'єкт без змін
    }
    
    Node* current = head.get();
    int count = 0;
    while (current) {
        if (count == AfterElem) {
            // Створення нового вузла з використанням make_unique
            std::unique_ptr<Node> newNode = std::make_unique<Node>(Node{ value, nullptr });
            
            // Присвоєння next вузлу newNode
            current->next = std::move(newNode);
            
            Size++;
            break;
        }
        current = current->next.get();
        count++;
    }
    return *this;
}

int CSingleLinkedList::getSize() const {
    return Size;
}

double  CSingleLinkedList::getMaxValue() const {
    if (head == nullptr) {
        return 0;
    }
    else {
        Node* current = head->next.get();
        double maxValue = head->data;
        while (current) {
            if (current->data > maxValue) {
                maxValue = current->data;
            }
            current = current->next.get();
        }
        return maxValue;
    }
}

double CSingleLinkedList::getMinValue() const {
    if (head == nullptr) {
        return 0;
    }
    else {
        Node* current = head->next.get();
        double minValue = head->data;
        while (current) {
            if (current->data < minValue) {
                minValue = current->data;
            }
            current = current->next.get();
        }
        return minValue;
    }
}
void CSingleLinkedList::selectionSorted(bool methodSort) {
    Node* i, * j;
    double temp;
    for (i = head.get(); i != nullptr; i = i->next.get()) {
        for (j = i->next.get(); j != nullptr; j = j->next.get()) {
            if (methodSort) { // true - сортування від меншого до більшого
                if (i->data > j->data) {
                    temp = i->data;
                    i->data = j->data;
                    j->data = temp;
                }
            }
            else {
                if (i->data < j->data) {
                    temp = i->data;
                    i->data = j->data;
                    j->data = temp;
                }
            }
        }
    }
}




//ОПЕРАТОРИ
CSingleLinkedList CSingleLinkedList::operator+(const CSingleLinkedList& data) {
    if (!head)return data;
        CSingleLinkedList result(*this);
        Node* currentHead = result.head.get();
        while (currentHead->next) {
            currentHead = currentHead->next.get();
        }
        
        Node* current = data.head.get();
        while (current) {
            currentHead->next = std::make_unique<Node>();
            currentHead->next->data = current->data;
            currentHead = currentHead->next.get();
            current = current->next.get();
        }

    result.Size += data.Size;
    
    return result;
}

CSingleLinkedList CSingleLinkedList::operator-(const CSingleLinkedList& data) {
    CSingleLinkedList result(*this);
    Node* currentData = data.head.get();
    while (currentData) {
        // Починаємо пошук поточного елемента списку у списку result
        Node* currentResult = result.head.get();
        Node* prevResult = nullptr;
        
        while (currentResult) {
            // Якщо знайдено співпадіння, видаляємо поточний елемент з result
            if (currentResult->data == currentData->data) {
                if (prevResult) {
                    prevResult->next = std::move(currentResult->next);
                }
                else {
                    result.head = std::move(currentResult->next);
                }
                result.Size--; 
                break;
            }
            
            prevResult = currentResult;
            currentResult = currentResult->next.get();
        }
        
        currentData = currentData->next.get();
    }
    
    return result;
}


CSingleLinkedList CSingleLinkedList::operator*(double scalar) {
    CSingleLinkedList result(*this);
    
    Node* current = result.head.get();
    while (current) {
        current->data *= scalar;
        current = current->next.get(); 
    }
    return result;
}

std::istream& operator>>(std::istream& input, CSingleLinkedList& list) {
    double value;
    while (input >> value) {
        std::unique_ptr<CSingleLinkedList::Node> newNode = std::make_unique<CSingleLinkedList::Node>(CSingleLinkedList::Node{ value, nullptr });
        
        if (list.head == nullptr) {
            list.head = std::move(newNode);
        }
        else {
            CSingleLinkedList::Node* current = list.head.get();
            while (current->next != nullptr) {
                current = current->next.get();
            }
            current->next = std::move(newNode);
        }
        
        list.Size++;
    }
    return input;
}

std::ostream& operator<<(std::ostream& output, const CSingleLinkedList& list) {
    CSingleLinkedList::Node* current = list.head.get();
    while (current) {
        output << current->data;
        current = current->next.get();
        if (current != nullptr) {
            output << " ";
        }
    }
    return output;
    
}

double  CSingleLinkedList::operator[](int index) {
    Node* current = head.get();
    int currentIndex = 0;
    while (current) {
        if (currentIndex == index) {
            return current->data;
        }
        current = current->next.get();
        currentIndex++;
    }
    return 0;
}
